
#define LED_PIN         13
#define BMA180_ADDRESS  0x80
#define I2C_SPEED       100000L

char data[512]; int i; // Data link
uint8_t rawADC[7];

#define toINT(LSByte, MSByte) ((int) ((unsigned int) (MSByte) << 8) | ((unsigned char)(LSByte)))

void waitTransmissionI2C() {
  uint16_t count = 255; while(!(TWCR & (1 << TWINT))) {count--; if(count == 0) {TWCR = 0; break;}}
}

void i2c_rep_start(uint8_t address) {
  TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN); waitTransmissionI2C();
  TWDR = address; TWCR = (1 << TWINT) | (1 << TWEN); waitTransmissionI2C();
}

void i2c_write(uint8_t data) {
  TWDR = data; TWCR = (1 << TWINT) | (1 << TWEN); waitTransmissionI2C();
}

uint8_t i2c_readAck() {
  TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWEA); waitTransmissionI2C();
  return TWDR;
}

uint8_t i2c_readNak() {
  TWCR = (1 << TWINT) | (1 << TWEN); waitTransmissionI2C();
  uint8_t r = TWDR; TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO); //i2c_stop
  return r;
}

void i2c_writeReg(uint8_t add, uint8_t reg, uint8_t val) {
  i2c_rep_start(add + 0); i2c_write(reg); i2c_write(val);
  TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO); //i2c_stop
}

uint8_t i2c_readReg(uint8_t add, uint8_t reg) {
  i2c_rep_start(add + 0); i2c_write(reg); i2c_rep_start(add + 1);
  return i2c_readNak();
}

void init_imu() {
  PORTC |= 1 << 4; PORTC |= 1 << 5;
  TWSR = 0; TWBR = ((16000000L / I2C_SPEED) - 16) / 2; TWCR = 1 << TWEN;
  delay(10); i2c_writeReg(BMA180_ADDRESS, 0x0D, 1 << 4);
  delay(5); uint8_t control = i2c_readReg(BMA180_ADDRESS, 0x20);
  control &= 0x0F; control |= (0x01 << 4); i2c_writeReg(BMA180_ADDRESS, 0x20, control);
  delay(5); control = i2c_readReg(BMA180_ADDRESS, 0x30);
  control &= 0xFC; control |= 0x00; i2c_writeReg(BMA180_ADDRESS, 0x30, control);
  delay(5); control = i2c_readReg(BMA180_ADDRESS, 0x35);
  control &= 0xF1; control |= (0x05 << 1); i2c_writeReg(BMA180_ADDRESS, 0x35, control);
  delay(5); ADCSRA |= _BV(ADPS2); ADCSRA &= ~_BV(ADPS1); ADCSRA &= ~_BV(ADPS0);
}

void read_imu() {
  TWBR = ((16000000L / 400000L) - 16) / 2;
  i2c_rep_start(BMA180_ADDRESS); i2c_write(0x02); i2c_rep_start(BMA180_ADDRESS + 1);
  rawADC[0] = i2c_readAck(); rawADC[1] = i2c_readAck();
  rawADC[2] = i2c_readAck(); rawADC[3] = i2c_readAck();
  rawADC[4] = i2c_readAck(); rawADC[5] = i2c_readAck();
  rawADC[6] = i2c_readNak(); // Temperature
}

void setup() {
  pinMode(LED_PIN, OUTPUT);
  Serial.begin(38400);
  init_imu();
  i = 0;
}

void loop() {
  digitalWrite(LED_PIN, HIGH);
  if (Serial.available() > 0) {
    data[i++] = Serial.read();
    if ((i >= 512) || (data[i - 1] == ';')) {
      if (data[0] == 'I') {
        // Process INIT block
      } else if (data[0] == 'D') {
        
        // Process DATA block
        digitalWrite(LED_PIN, LOW);
        Serial.print("CTRL,");
        for(i = 5; (data[i] != ',') && (data[i] != ';') && (i < 512); i++) Serial.print(data[i]);
        Serial.print(",0,1,1,");
        
        // Include the vector control
        read_imu(); delay(10);
        Serial.print(toINT(rawADC[0], rawADC[1]) >> 2); Serial.print(",");
        Serial.print(toINT(rawADC[2], rawADC[3]) >> 2); Serial.print(",");
        Serial.print(toINT(rawADC[4], rawADC[5]) >> 2); Serial.print(";");
      }
      i = 0;
    }
  }
}

