#include <Wire.h>
//#include "itg3200.h" // GYRO ITG-3200
#define GyroID  B1101001
//Register 22 - FS_SEL, DLPF_CFG
#define R22     0x16
#define R22INIT B00011011
//Register 62 - Power Management
#define R62     0x3e
#define R62INIT B00000001
//Registers 27 to 34 - Sensor Registers
#define TempH   0x1b
#define TempL   0x1c
#define GyroXH  0x1d
#define GyroXL  0x1e
#define GyroYH  0x1f
#define GyroYL  0x20
#define GyroZH  0x21
#define GyroZL  0x22

//#include "lis331hh.h" Accelerometer LIS331HH
#define AccID   0x18
//Register CTRL_REG0 - Power management and conf
#define RCTR0   0x20
#define RCTR0INIT B00100111
//Sensor Registers
#define AccXH   0x29
#define AccXL   0x28
#define AccYH   0x2B
#define AccYL   0x2A
#define AccZH   0x2D
#define AccZL   0x2C
#define AccStat 0x27

void i2cSetRegister(int device, int address, int value) {
  Wire.beginTransmission(device);
  Wire.send(address);
  Wire.send(value);
  Wire.endTransmission();
}

byte i2cGetRegister(int device, int address) {
  Wire.beginTransmission(device);
  Wire.send(address);
  Wire.endTransmission();
  Wire.requestFrom(device, 1);
  if(Wire.available())
    return Wire.receive();
  return B00000000;
}

int i2cGetValue(int device, int addressH, int addressL) {
  return i2cGetRegister(device, addressH) * 256 + i2cGetRegister(device, addressL);
}

// Globals
int ledPin = 13;
int i;
char a[512];

void setup()
{
  Wire.begin();
  // Hyperterminal at 57600 and Arduino at 115200 bauds
  Serial.begin(115200);
  // Accelerometer LIS331HH setup
  i2cSetRegister(AccID, RCTR0, RCTR0INIT);
  // Gyro ITG-3200 setup
  i2cSetRegister(GyroID, R62, R62INIT);
  i2cSetRegister(GyroID, R22, R22INIT);
  // Hardware-in-the-loop config
  i = 0;
}

void loop()
{
  double t, u, v, w;
  int m, n, o, p;

  digitalWrite(ledPin, HIGH);
  if (Serial.available() > 0)
    a[i++] = Serial.read();
  if ((i >= 512) || (a[i-1] == ';')) {
    if (a[0] == 'I') {
      // Process INIT block

    } else if (a[0]=='D') {
      // Look for the time stamp
      m = 4;
      for(n = m + 1; (n < 512) && (a[n] != ',') && (a[n] != ';'); n++);

      // Process DATA block
      digitalWrite(ledPin, LOW);
      Serial.print("CTRL,"); o = 4;
      for(p = m + 1; p <= (n - 1); p++, o++)
        Serial.print(a[p]);
      Serial.print(",1,1,1,"); o += 13;
      w = i2cGetValue(AccID, AccXH, AccXH);
      u = i2cGetValue(AccID, AccYH, AccYH);
      v = i2cGetValue(AccID, AccZH, AccZH);
      t = sqrt(u * u + v * v + w * w); // Normalize
      if(t < 1E-45) {u = 0; v = 0; w = 0;}
      else {u /= t; v /= t; w /= t;}
      Serial.print(u);
      Serial.print(",");
      Serial.print(v);
      Serial.print(",");
      Serial.print(w);
      Serial.print(";");
    }
    i = 0;
  }
}

